'use strict';

var chalk = require('chalk');

var _require = require('./utils'),
    getAST = _require.getAST,
    getClassNames = _require.getClassNames;

var getGlamorStyleSheet = function () {
  return require('glamor').styleSheet;
};

/*
 * Taken from
 * https://github.com/facebook/jest/blob/be4bec387d90ac8d6a7596be88bf8e4994bc3ed9/packages/expect/src/jasmine_utils.js#L234
 */
function isA(typeName, value) {
  return Object.prototype.toString.apply(value) === `[object ${typeName}]`;
}

/*
 * Taken from
 * https://github.com/facebook/jest/blob/be4bec387d90ac8d6a7596be88bf8e4994bc3ed9/packages/expect/src/jasmine_utils.js#L36
 */
function isAsymmetric(obj) {
  return obj && isA('Function', obj.asymmetricMatch);
}

function valueMatches(declaration, value) {
  if (value instanceof RegExp) {
    return value.test(declaration.value);
  }

  if (isAsymmetric(value)) {
    return value.asymmetricMatch(declaration.value);
  }

  return value === declaration.value;
}

function toHaveStyleRule(received, property, value) {
  var selectors = getClassNames(received);
  var ast = getAST(selectors, getGlamorStyleSheet());

  var classNames = getClassNames(received);
  var rules = ast.stylesheet.rules.filter(function (rule) {
    return classNames.some(function (cn) {
      return rule.selectors.includes(`.${cn}`);
    });
  });

  var declaration = rules.reduce(function (decs, rule) {
    return Object.assign([], decs, rule.declarations);
  }, []).filter(function (dec) {
    return dec.type === 'declaration' && dec.property === property;
  }).pop();

  if (!declaration) {
    return {
      pass: false,
      message: function message() {
        return `Property not found: ${property}`;
      }
    };
  }

  var pass = valueMatches(declaration, value);

  return {
    pass,
    message: function message() {
      return `Expected ${property}${pass ? ' not ' : ' '}to match:\n` + `  ${chalk.green(value)}\n` + 'Received:\n' + `  ${chalk.red(declaration.value)}`;
    }
  };
}

module.exports = {
  toHaveStyleRule
};